<?php
/*
   This include() script adds missing PHP functions to earlier interpreter
   versions, so you can make downwards compatible scripts without having
   to stick to the least common denominator. It only defines the ones that
   are really missing; native functions will so get used, where available.

   - many of the emulation functions are one-liners
   - a few features have been added that never made it into one of the
     official versions (CVS code and the ever-absent "gzdecode" and
     "file_put_contents" for example)
   - many things are still missing, and external add-ons are provided to
     mimic certain other PHP core extension modules
   - a few very extravagant functions (array_u?diff*_u*assoc?) have been
     separated out into ext/
   - the extended OO-capabilities and language syntax extensions of PHP5
     and ZE2 cannot seriously be emulated here, this script only takes care
     about procedural interfaces
   - this script has itself PHP 4.1 as minimum version requirement
   - this is PuplicDomain (no copyright, no license, no warranty) so you
     can melt it into anything, regardless of your preferred license (you
     may strip this paragraph and turn it all into GPL, BSD, GNU LGPL,
     Artistic, MPL, PHP license, M$ EULA, or whatever you like best)
   
   Get update notes via "http://freshmeat.net/projects/upgradephp" or
   google for it. Any contribution is appreciated. <milky*userssfnet>
*/





#------------------------------------------------------------------ CVS ---
// most of this appeared in 5.0
// ...




#------------------------------------------------------------- optional ---
// following functions aren't present everywhere (regardless of version)
// ... (should probably go into ext/ then?)




#------------------------------------------------------------------ 6.0 ---
// followig functions were never implemented in PHP


#-- inflates a string enriched with gzip headers
#   (this is the logical counterpart to gzencode(), but don't tell anyone!)
if (!function_exists("gzdecode")) {
   function gzdecode($data, $maxlen=NULL) {

      #-- decode header
      $len = strlen($data);
      if ($len < 20) {
         return;
      }
      $head = substr($data, 0, 10);
      $head = unpack("n1id/C1cm/C1flg/V1mtime/C1xfl/C1os", $head);
      list($ID, $CM, $FLG, $MTIME, $XFL, $OS) = array_values($head);
      $FTEXT = 1<<0;
      $FHCRC = 1<<1;
      $FEXTRA = 1<<2;
      $FNAME = 1<<3;
      $FCOMMENT = 1<<4;
      $head = unpack("V1crc/V1isize", substr($data, $len-8, 8));
      list($CRC32, $ISIZE) = array_values($head);

      #-- check gzip stream identifier
      if ($ID != 0x1f8b) {
         trigger_error("gzdecode: not in gzip format", E_USER_WARNING);
         return;
      }
      #-- check for deflate algorithm
      if ($CM != 8) {
         trigger_error("gzdecode: cannot decode anything but deflated streams", E_USER_WARNING);
         return;
      }

      #-- start of data, skip bonus fields
      $s = 10;
      if ($FLG & $FEXTRA) {
         $s += $XFL;
      }
      if ($FLG & $FNAME) {
         $s = strpos($data, "\000", $s) + 1;
      }
      if ($FLG & $FCOMMENT) {
         $s = strpos($data, "\000", $s) + 1;
      }
      if ($FLG & $FHCRC) {
         $s += 2;  // cannot check
      }
      
      #-- get data, uncompress
      $data = substr($data, $s, $len-$s);
      if ($maxlen) {
         $data = gzinflate($data, $maxlen);
         return($data);  // no checks(?!)
      }
      else {
         $data = gzinflate($data);
      }
      
      #-- check+fin
      $chk = crc32($data);
      if ($CRC32 != $chk) {
         trigger_error("gzdecode: checksum failed (real$chk != comp$CRC32)", E_USER_WARNING);
      }
      elseif ($ISIZE != strlen($data)) {
         trigger_error("gzdecode: stream size mismatch", E_USER_WARNING);
      }
      else {
         return($data);
      }
   }
}




#------------------------------------------------------------------ 5.0 ---
# set_exception_handler - unimpl.
# restore_exception_handler - unimpl.
# debug_print_backtrace - unimpl.
# class_implements - unimpl.
# proc_terminate - unimpl?
# proc_get_status - unimpl.
# --
# proc_nice
# setrawcookie - only via header() ? - or unclean setcookie(...,urldecode($2),...) ?
# dns_get_record
# date_sunrise - undoc.
# date_sunset - undoc.


#-- ci string search functions
if (!function_exists("stripos")) {
   #-- find position of first occourence of a case-insensitive string
   function stripos($haystack, $needle, $offset=NULL) {
      return strpos(strtolower($haystack), strtolower($needle), $offset);
   }
}

#-- more complicated, because $offset param not supported in basic form
if (!function_exists("strripos")) {
   #-- ... from end of string
   function strripos($haystack, $needle, $offset=NULL) {
      if (isset($offset)) {
         $haystack = strtolower($haystack);
         $needle = strtolower($needle);
         if ($offset < 0) {
            $offset = strlen($haystack) + $offset;
         }
         do {
            $l = $new;
            $new = strpos($haystack, $needle, $l);
         }
         while (($new !== false) && ($new < $offset));
         return($l);
      }
      else {
         return strrpos(strtolower($haystack), strtolower($needle));
      }
   }
}


#-- case-insensitive version of str_replace
if (!function_exists("str_ireplace")) {
   function str_ireplace($search, $replace, $subject, $count=NULL) {
      if (is_array($search)) {
         $replace = array_values($replace);
         foreach (array_values($search) as $i=>$srch) {
            $subject = str_ireplace($srch, $replace[$i], $subject);
         }
      }
      else {
         $search = "\007" . preg_quote($search) . "\007i";
         $replace = strtr($replace, array('$'=>'\\$', "\\"=>"\\\\"));
         $subject = preg_replace($search, $replace, $subject);
      }
      return($subject);
   }
}


#-- performs a http HEAD request
if (!function_exists("get_headers")) {
   function get_headers($url, $parse=0) {
      $c = parse_url($url);
      extract($c);
      if (!isset($port)) { 
         $port = 80;
      }
      if ($f = fsockopen($host, $port, $errno, $errstr, $timeout=5)) {
         fwrite($f, "HEAD $path HTTP/1.1\015\012"
                  . "Host: $host\015\012"
                  . "Connection: close\015\012"
                  . "Accept: text/html, application/xml, text/xml;q=0.75, application/xhtml+xml, text/plain;q=0.5, */*;q=0.01\015\012"
                  . "User-Agent: ".trim(ini_get("user_agent"))."\015\012"
                  . "\015\012");
         socket_set_blocking($f, true);
         $ls = array();
         while (!feof($f)) {
            $line = fgets($f, 1<<16);
            if (!($line = trim($line))) {
               break;
            }
            elseif ($parse) {
               if ($l = strpos($line, ":")) {
                  $name = substr($line, 0, $l);
                  $value = trim(substr($line, $l + 1));
                  if (isset($ls[$name])) {
                     $ls[$name] .= ", $value";
                  }
                  else {
                     $ls[$name] = $value;
                  }
               }
               else {
                  $ls[] = $line;
               }
            }
            else {
               $ls[] = $line;
            }
         }
         fclose($f);
         return($ls);
      }
      else {
         return(false);
      }
   }
}


#-- stub
if (!function_exists("headers_list")) {
   function headers_list() {
      trigger_error("headers_list: not supported by this PHP version", E_USER_WARNING);
      return (array)NULL;
   }
}


#-- write formatted string to stream/file
if (!function_exists("fprintf")) {
   function fprintf() {
      $args = func_get_args();
      $stream = array_shift($args);
      return fwrite($stream, call_user_func_array("sprintf", $args));
   }
}
if (!function_exists("vfprintf")) {
   function vfprintf($stream, $format, $args=NULL) {
      return fwrite($stream, vsprintf($format, $args));
   }
}


#-- splits a string in even sized chunks, returns an array
if (!function_exists("str_split")) {
   function str_split($str, $chunk=1) {
      $r = array();
      if ($chunk < 1) {
         $r[] = $str;
      }
      else {
         $len = strlen($str);
         for ($n=0; $n<$len; $n+=$chunk) {
            $r[] = substr($str, $n, $chunk);
         }
      }
      return($r);
   }
}


#-- mmmh?
if (!function_exists("http_build_query")) {
   function http_build_query($data, $prefix="", $fpfix="", $l=0) {
      $s = "";
      foreach ($data as $in=>$val) {
         $s_in = $in;
         if ($fpfix) {
            $s_in = $l ? "$fpfix"."[$s_in]" : "$fpfix$s_in";
         }
         elseif (is_int($s_in) && $prefix) {
            $s_in = "$prefix$s_in";
         }
         if (is_array($val)) {
            $s .= http_build_query($val, "", $s_in, $l+1);
         }
         else {
            $s .= "&$s_in=" . urlencode($val);
         }
      }
      if (!$fpfix) { $s = substr($s, 1); }
      return($s);
   }
}


#-- transform into 3to4 uuencode (bare encoding, not the uu file format)
if (!function_exists("convert_uuencode")) {
   function convert_uuencode($data) {
      $out = "";
      $line = "";
      $len = strlen($data);
      $data .= "\252\252\252";   // PHP and uuencode(1) use some special garbage??
      for ($n=0; $n<$len; ) {
         $x = (ord($data[$n++]) << 16)
            + (ord($data[$n++]) <<  8)
            + (ord($data[$n++]) <<  0);
         $line .= chr( 32 + (($x >> 18) & 0x3f) )
               . chr( 32 + (($x >> 12) & 0x3f) )
               . chr( 32 + (($x >>  6) & 0x3f) )
               . chr( 32 + (($x >>  0) & 0x3f) );
         if (($n % 45) == 0) {
            $out .= chr(32+45) . "$line\n";
            $line = "";
         }
      }
      if ($trail = ($len % 45)) {
         $out .= chr(32 + $trail) . "$line\n";
      }
      return($out);
   }
}

#-- decodes again what the above mangled
if (!function_exists("convert_uudecode")) {
   function convert_uudecode($data) {
      $out = "";
      foreach(explode("\n", ltrim($data)) as $line) {
         if (!strlen($line)) {
            break;
         }
         unset($num);
         $num = ord($line{0}) - 32;
         if (($num <= 0) || ($num > 62)) {  // 62 is the maximum line length
            break;          // according to uuencode(5), so we stop here too
         }
         $line = substr($line, 1);
         $add = "";
         for ($n=0; strlen($add)<$num; ) {
            $x = ((ord($line[$n++]) - 32) << 18)
               + ((ord($line[$n++]) - 32) << 12)
               + ((ord($line[$n++]) - 32) <<  6)
               + ((ord($line[$n++]) - 32) <<  0);
            $add .= chr( ($x >> 16) & 0xff )
                  . chr( ($x >>  8) & 0xff )
                  . chr( ($x >>  0) & 0xff );
         }
         $out .= substr($add, 0, $num);   // cut any trailing garbage (last two decoded chars may be wrong)
         $line="";
      }
      return($out);
   }
}


#-- return array of filenames in a given directory (only works for local files)
if (!function_exists("scandir")) {
   function scandir($dirname, $desc=0) {
      if (strpos($dirname, "file://") === 0) {
         $dirname = substr($dirname, 7);
         if (strpos($dirname, "localh") === 0) {
            $dirname = substr($dirname, strpos($dirname, "/"));
         }
      }
      if ($dh = opendir($dirname)) {
         $ls = array();
         while ($fn = readdir($dh)) {
            $ls[] = $fn;
         }
         closedir($dh);
         if ($desc) {
            rsort($ls);
         }
         else {
            sort($ls);
         }
         return $ls;
      }
      else {
         return false;
      }
   }
}


#-- like date(), but returns an integer for given one-letter format parameter
if (!function_exists("idate")) {
   function idate($formatchar, $timestamp=NULL) {
      if (strlen($formatchar) != 1) {
         return false;
      }
      if (!isset($timestamp)) {
         $timestamp = time();
      }
      return((int)(date($formatchar, $timestamp)));
   }
}



#-- combined sleep() and usleep() 
if (!function_exists("time_nanosleep")) {
   function time_nanosleep($sec, $nano) {
      sleep($sec);
      usleep($nano);
   }
}



#-- search first occourence of any of the given chars, returns rest of haystack
#   (char_list must be a string for compatibility with the real PHP func)
if (!function_exists("strpbrk")) {
   function strpbrk($haystack, $char_list) {
      $cn = strlen($char_list);
      $min = strlen($haystack);
      for ($n=0; $n<$cn; $n++) {
         $l = strpos($haystack, $char_list{$n});
         if (($l !== false) && ($l < $min)) {
            $min = $l;
         }
      }
      return($min ? substr($haystack, $min) : $haystack);
   }
}



#-- logo image activation URL query strings
if (!function_exists("php_real_logo_guid")) {
   function php_real_logo_guid() {
      return php_logo_guid();
   }
   function php_egg_logo_guid() {
      return zend_logo_guid();
   }
}


#-- calls PHP interpreter itself (really only works with 4.3 onwards)
if (!function_exists("php_strip_whitespace")) {
   function php_strip_whitespace($fn) {
      $fn = escapeshellcmd($fn);
      $text = `php -wqCf '$fn'`;
      if (!$text) {
         $text = implode("", file($fn));
      }
      return $text;
   }
}

#-- innovocates PHP interpreter to do the syntax check (nothing else can do)
if (!function_exists("php_check_syntax")) {
   function php_check_syntax($fn) {
      $args = func_get_args();
      if (count($args)>1) {
         $result = & $args[1];
      }
      $fn = escapeshellcmd($fn);
      $result = system("php -lqCf '$fn'", $err);
      return($err==0);
   }
}


#-- no need to implement this (there aren't interfaces in PHP4 anyhow)
if (!function_exists("get_declared_interfaces")) {
   function get_declared_interfaces() {
      trigger_error("get_declared_interfaces: Current script won't run reliably with PHP4.", E_USER_WARNING);
      return( (array)NULL );
   }
}


#-- creates an array from lists of $keys and $values (should have same number of entries)
if (!function_exists("array_combine")) {
   function array_combine($keys, $values) {
      $keys = array_values($keys);
      $values = array_values($values);
      $r = array();
      foreach ($values as $i=>$val) {
         if ($key = $keys[$i]) {
            $r[$key] = $val;
         }
         else {
            $r[] = $val;
         }
      }
      return($r);
   }
}


#-- apply userfunction to each array element (descending recursively)
#   use it like:  array_walk_recursive($_POST, "stripslashes");
if (!function_exists("array_walk_recursive")) {
   function array_walk_recursive(&$input, $callback, $userdata=NULL) {

      #-- walk
      foreach ($input as $i=>$value) {
         #-- recurse
         if (is_array($value)) {
            array_walk_recursive($input[$i], $callback, @$userdata);
         }
         #-- scalar
         else {
            call_user_func_array($callback, array(&$input[$i], &$i, @$userdata) );
         }
      }
      return NULL; //?
   }
}


#-- complicated wrapper around substr() and and strncmp()
if (!function_exists("substr_compare")) {
   function substr_compare($haystack, $needle, $offset=0, $len=0, $ci=0) {

      #-- check params   
      if ($len <= 0) {   // not well documented
         $len = strlen($needle);
         if (!$len) { return(0); }
      }
      #-- length exception
      if ($len + $offset >= strlen($haystack)) {
         trigger_error("substr_compare: given length exceeds main_str", E_USER_WARNING);
         return(false);
      }
      #-- cut
      if ($offset) {
         $haystack = substr($haystack, $offset, $len);
      }
      #-- case-insensitivity
      if ($ci) {
         $haystack = strtolower($haystack);
         $needle = strtolower($needle);
      }

      #-- do
      return(strncmp($haystack, $needle, $len));
   }
}


#-- stub, returns empty list as usual
if (!function_exists("spl_classes")) {
   function spl_classes() {
      trigger_error("spl_classes: not built into this PHP version");
      return (array)NULL;
   }
}



#-- gets you list of class names the given objects class was derived from, slow
if (!function_exists("class_parents")) {
   function class_parents($obj) {
      $all = get_declared_classes();
      $r = array();
      foreach ($all as $potential_parent) {
         if (is_subclass_of($obj, $potential_parent)) {
            $r[$potential_parent] = $potential_parent;
         }
      }
      return($r);
   }
}


#-- an alias
if (!function_exists("session_commit") && function_exists("session_write_close")) {
   function session_commit() {
      session_write_close();
   }
}


#-- aliases
if (!function_exists("dns_check_record")) {
   function dns_check_record($host, $type=NULL) {
      return checkdnsrr($host, $type);
   }
}
if (!function_exists("dns_get_mx")) {
   function dns_get_mx($host, $mx) {
      $args = func_get_args();
      if ($args[2]) { $w = & $args[2]; }
      return getmxrr($host, $mx, $w);
   }
}









#------------------------------------------------------------------ 4.3 ---
# money_format - unimpl?
# sha1
# sha1_file


#-- simplified file read-at-once function
if (!function_exists("file_get_contents")) {
   function file_get_contents($filename, $use_include_path=1) {
      if ($f = fopen($filename, "rb", $use_include_path)) {
         $content = fread($f, 1<<19);  # max 512K
         fclose($f);
         return($content);
      }
   }
}

#-- the write-at-once equivalent
if (!function_exists("file_put_contents")) {
   function file_put_contents($filename, $data, $flags=0, $resource=NULL) {
      if ($f = @fopen($filename, ($flags&FILE_APPEND?"a":"w")."b", $flags&FILE_USE_INCLUDE_PATH, $resource)) {
         fwrite($f, $data);
         fclose($f);
         return(TRUE);
      }
   }
}


#-- shell-like filename matching
if (!function_exists("fnmatch")) {
   function fnmatch($pattern, $str, $flags=0x0000) {
      static $cmp = array();
      $rx = $cmp["$pattern$flags"];
      if (!$rx) {
         if (count($cmp) >= 10) { $cmp = array(); }
         $rx = preg_quote($pattern);
         $rx = strtr($rx, array(
            "\\*"=>".*?", "\\?"=>".", "\\["=>"[", "\\]"=>"]",
         ));
         $rx = "<^$rx$>" . ($flags&GLOB_NOCASE?"i":"");
         $cmp["$pattern$flags"] = $rx;
      }
      return(preg_match($rx, $str));
   }
}


#-- file search and name matching (with shell patterns)
if (!function_exists("glob")) {
   define("GLOB_MARK", 1<<0);
   define("GLOB_NOSORT", 1<<1);
   define("GLOB_NOCHECK", 1<<2);
   define("GLOB_NOESCAPE", 1<<3);
   define("GLOB_BRACE", 1<<4);
   define("GLOB_ONLYDIR", 1<<5);
   define("GLOB_NOCASE", 1<<6);
   define("GLOB_DOTS", 1<<7);
   // unlikely to work under Win(?), without replacing the explode() with
   // a preg_split() incorporating the native DIRECTORY_SEPARATOR as well

   function glob($pattern, $flags=0x0000) {
      $ls = array();
      $rxci = ($flags & GLOB_NOCASE) ? "i" : "";
#echo "\n=> glob($pattern)...\n";
      
      #-- transform glob pattern into regular expression
      if ($pattern) {

         #-- look at each directory/fn spec part separately
         $parts2 = explode("/", $pattern);
         $pat = preg_quote($pattern);
         $pat = strtr($pat, array("\\*"=>".*?", "\\?"=>".?"));
         if ($flags ^ GLOB_NOESCAPE) {
            // uh, oh, ouuch - the above is unclean enough...
         }
         if ($flags ^ GLOB_BRACE) {
            $pat = preg_replace("/\{(.+?)\}/e", 'strtr("[$1]", ",", "")', $pat);
         }
         $parts = explode("/", $pat);
#echo "parts == ".implode(" // ", $parts) . "\n";
         $lasti = count($parts) - 1;
         $dn = "";
         foreach ($parts as $i=>$p) {

            #-- basedir included (yet no pattern matching necessary)
            if (!strpos($p, "*?") && (strpos($p, ".?")===false)) {
               $dn .= $parts2[$i] . ($i!=$lasti ?"/" :"");
#echo "skip:$i, cause no pattern matching char found -> only a basedir spec\n";
               continue;
            }
            
            #-- start reading dir + match filenames against current pattern
            if ($dh = opendir($dn ?$dn:'.')) {
               $with_dot = ($p[1]==".") || ($flags & GLOB_DOTS);
#echo "part:$i:$p\n";
#echo "reading dir \"$dn\"\n";
               while ($fn = readdir($dh)) {
                  if (preg_match("\007^$p$\007$rxci", $fn)) {

                     #-- skip over 'hidden' files
                     if (($fn[0] == ".") && !$with_dot) {
                        continue;
                     }

                     #-- add filename only if last glob/pattern part
                     if ($i==$lasti) {
                        if (is_dir("$dn$fn")) {
                           if ($flags & GLOB_ONLYDIR) {
                              continue;
                           }
                           if ($flags & GLOB_MARK) {
                              $fn .= "/";
                           }
                        }
#echo "adding '$fn' for dn=$dn to list\n";
                        $ls[] = "$dn$fn";
                     }

                     #-- initiate a subsearch, merge result list in
                     elseif (is_dir("$dn$fn")) {
                        // add reamaining search patterns to current basedir
                        $remaind = implode("/", array_slice($parts2, $i+1));
                        $ls = array_merge($ls, glob("$dn$fn/$remaind", $flags));
                     }
                  }
               }
               closedir($dh);

               #-- prevent scanning a 2nd part/dir in same glob() instance:
               break;  
            }

            #-- given dirname doesn't exist
            else {
               return($ls);
            }

         }// foreach $parts
      }

      #-- return result list
      if (!$ls && ($flags & GLOB_NOCHECK)) {
         $ls[] = $pattern;
      }
      if ($flags ^ GLOB_NOSORT) {
         sort($ls);
      }
#print_r($ls);
#echo "<=\n";
      return($ls);
   }
}


#-- redundant alias for isset()
if (!function_exists("array_key_exists")) {
   function array_key_exists($key, $search) {
      return isset($search[$key]);
   }
}


#-- who could need that?
if (!function_exists("array_intersect_assoc")) {
   function array_intersect_assoc( /*array, array, array...*/ ) {
      $whatsleftover = array();
      $in = func_get_args();
      $cmax = count($in);
      foreach ($in[0] as $i=>$v) {
         for ($c=1; $c<$cmax; $c++) {
            if (@$in[$c][$i] !== $v) {
               continue 2;
            }
         }
         $whatsleftover[$i] = $v;
      }
      return $whatsleftover;
   }
}


#-- the opposite of the above
if (!function_exists("array_diff_assoc")) {
   function array_diff_assoc( /*array, array, array...*/ ) {
      $diff = array();
      $in = func_get_args();
      foreach ($in[0] as $i=>$v) {
         for ($c=1; $c<count($in); $c++) {
            if (isset($in[$c][$i]) && ($in[$c][$i] == $v)) {
               continue 2;
            }
         }
         $diff[$i] = $v;
      }
      return $diff;
   }
}


#-- opposite of htmlentities
if (!function_exists("html_entity_decode")) {
   function html_entity_decode($string, $quote_style=ENT_COMPAT, $charset="ISO-8859-1") {
      // we fall short on anything other than Latin-1
      $y = array_flip(get_html_translation_table(HTML_ENTITIES, $quote_style));
      return strtr($string, $y);
   }
}


#-- extracts single words from a string
if (!function_exists("str_word_count")) {
   function str_word_count($string, $format=0) {
      preg_match_all('/([\w](?:[-\'\w]?[\w]+)*)/', $string, $uu);
      if (!$format) {
         return(count($uu[1]));
      }
      elseif ($format==1) {
         return($uu[1]);
      }
      else {
         $r = array();
         $l = 0;
         foreach ($uu[1] as $word) {
            $l = strpos($string, $word, $l);
            $r[$l] = $word;
            $l += strlen($word);
         }
         return($r);
      }
   }
}


#-- creates a permutation of the given strings characters
if (!function_exists("str_shuffle")) {
   function str_shuffle($str) {
      $r = "";
      while (strlen($str)) {
         $n = strlen($str) - 1;
         if ($n) {
            $n = rand(0, $n);
         }
         $r .= $str[$n];
         $str = substr($str, 0, $n) . substr($str, $n + 1);
      }
      return($r);
   }
}








#------------------------------------------------------------------ 4.2 ---
# - is_nan
# - is_infinite, is_finite


#-- shy away from this one - it was broken in all real PHP4.2 versions, and
#   this function emulation script won't change that
if (!function_exists("str_rot13")) {
   function str_rot13($str) {
      static $from = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
      static $to = "NOPQRSTUVWXYZABCDEFGHIJKLMnopqrstuvwxyzabcdefghijklm";
      return strtr($str, $from, $to);
   }
}


#-- well, if you need it
if (!function_exists("array_change_key_case")) {
   define("CASE_LOWER", 0);
   define("CASE_UPPER", 1);
   function array_change_key_case($array, $case=CASE_LOWER) {
      foreach ($array as $i=>$v) {
         unset($array[$i]);
         if (is_string($i)) {
            $i = ($case==CASE_LOWER) ? strtolower($i) : strtoupper($i);
         }
         $array[$i] = $v;
      }
      return($array);
   }
}


#-- hey, why not
if (!function_exists("array_fill")) {
   function array_fill($start_index, $num, $value) {
      $r = array();
      for ($i=$start_index,$end=$num+$i; $i<$end; $i++) {
         $r[$i] = $value;
      }
      return($r);
   }
}


#-- split an array into evenly sized parts
if (!function_exists("array_chunk")) {
   function array_chunk($input, $size, $preserve_keys=false) {
      $r = array();
      $n = -1;
      foreach ($input as $i=>$v) {
         if (($n < 0) || (count($r[$n]) == $size)) {
            $n++;
            $r[$n] = array();
         }
         if ($preserve_keys) {
            $r[$n][$i] = $v;
         }
         else {
            $r[$n][] = $v;
         }
      }
      return($r);
   }
}


#-- convenience wrapper
if (!function_exists("md5_file")) {
   function md5_file($filename, $raw_output=false) {
      if ($f = fopen($filename, "rb")) {
         $data = fread($f, 1<<22);  // can be too large for mem
         fclose($f);
         $r = md5($data);
         $data = NULL;
         if ($raw_output) {
            $r = pack("H*", $r);
         }
         return $r;
      }
   }
}


#-- object type checking
if (!function_exists("is_a")) {
   function is_a($obj, $classname) {
      return ((strtolower(get_class($obj))==strtolower($classname))
          or is_subclass_of($obj, $classname));
   }
}


#-- floating point modulo
if (!function_exists("fmod")) {
   function fmod($x, $y) {
      $r = $x / $y;
      $r -= (int)$r;
      $r *= $y;
      return($r);
   }
}








#------------------------------------------------------------------ 4.1 ---


#function nl_langinfo($item) {
#   $all = localeconv();
#   // unimplementable??
#}


#-- a silly alias (an early fallen attempt to unify PHP function names)
if (!function_exists("diskfreespace")) {
   function diskfreespace() {
      return disk_free_sapce();
   }
   function disktotalspace() {
      return disk_total_sapce();
   }
}


#-- variable count of arguments (in array list) printf variant
if (!function_exists("vprintf")) {
   function vprintf($format, $args=NULL) {
      call_user_func_array("fprintf", get_func_args());
   }
}


#-- same as above, but doesn't output directly and returns formatted string
if (!function_exists("vsprintf")) {
   function vsprintf($format, $args=NULL) {
      $args = array_merge(array($format), array_values((array)$args));
      return call_user_func_array("sprintf", $args);
   }
}



#-- end
// no need to implement anything below that, because such old versions
// will be incompatbile anyhow (- none of the newer superglobals known)

#---------------------------------------------------------------- 4.0.6 ---
#array_filter
#_map
#_merge

#---------------------------------------------------------------- 4.0.5 ---
# localeconv - UNIMPL!!
# array_reduce
# array_search
# strcoll

#---------------------------------------------------------------- 4.0.4 ---
# array_sum

#---------------------------------------------------------------- 4.0.3 ---
# is_uplo
# move_uploaded_file

#---------------------------------------------------------------- 4.0.2 ---
# strncasecmp
# wordwrap

#---------------------------------------------------------------- 4.0.1 ---
# levensth
# fflush() - UNIMPLementable!!!
# array_unique
# array_diff
# array_intersect
# sscanf
# str_pad


?>